import aiohttp
from goat.decorators.tool import Tool
from .parameters import JSONRpcBodyParameters

class JSONRpcService:
    def __init__(self, endpoint: str):
        self.endpoint = endpoint

    @Tool({
        "description": "Make a remote procedure call to a JSON RPC endpoint",
        "parameters_schema": JSONRpcBodyParameters
    })
    async def JSONRpcFunc(self, parameters: dict):
        """Makes a POST request to the configured endpoint with the required JSON-RPC parameters."""
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(self.endpoint, json=parameters) as response:
                    if not response.ok:
                        raise Exception(f"HTTP error! status: {response.status}, body: {await response.text()}")
                    return await response.json()
        except Exception as e:
            raise Exception(f"Failed to call {self.endpoint}: {e}")



from pydantic import BaseModel, Field
from typing import List

class JSONRpcBodyParameters(BaseModel):
    method: str = Field(
        ...,
        description="A string containing the name of the method to be invoked"
    )
    params: List[str] = Field(
        ...,
        description="A structured value that holds the parameter value to be used during the invokation of the method"
    )
    id: int = Field(
        ...,
        description="An identifier established by the client that must contain a string number or null"
    )
    jsonrpc: str = Field(
        ...,
        description="A string that specifies the version of the JSON-RPC protocol must be exactly '2.0'"
    )



from dataclasses import dataclass

from goat.classes.plugin_base import PluginBase
from .service import JSONRpcService


@dataclass
class JSONRpcPluginOptions:
    endpoint: str


class JSONRpcPlugin(PluginBase):
    def __init__(self, options: JSONRpcPluginOptions):
        super().__init__("jsonrpc", [JSONRpcService(options.endpoint)])

    def supports_chain(self, chain) -> bool:
        return True


def jsonrpc(options: JSONRpcPluginOptions) -> JSONRpcPlugin:
    return JSONRpcPlugin(options)
